@@ -57,28 +57,28 @@ class Context(object):
         >>> result = v.process({'a':1, 'b': 1}) #doctest: +ELLIPSIS
         <formalize.core.trace_DictValidator object at ...>
         {'currentfield': None,
+         'inputdata': {'a': 1, 'b': 1},
          'parent': None,
-         'rawdata': <formalize.core.trace_DictValidator object at ...>,
-         'validator': {'a': 1, 'b': 1},
+         'validator': <formalize.core.trace_DictValidator object at ...>,
          'workingdata': None}
         ----
         <formalize.core.trace_Int object at ...>
         {'currentfield': 'a',
+         'inputdata': {'a': 1, 'b': 1},
          'parent': <formalize.core.Context object at ...>,
-         'rawdata': {'a': 1, 'b': 1},
          'validator': <formalize.core.trace_DictValidator object at ...>,
          'workingdata': {}}
         ----
         <formalize.core.trace_Int object at ...>
         {'currentfield': 'b',
+         'inputdata': {'a': 1, 'b': 1},
          'parent': <formalize.core.Context object at ...>,
-         'rawdata': {'a': 1, 'b': 1},
          'validator': <formalize.core.trace_DictValidator object at ...>,
          'workingdata': {'a': {}}}
         ----
 
     """
-    def __init__(self, rawdata, validator=None, parent=None):
+    def __init__(self, validator, inputdata, parent=None):
 
         # The parent validator
         self.validator = validator
@@ -87,7 +87,7 @@ class Context(object):
         self.parent = parent
 
         # The unprocessed data
-        self.rawdata = rawdata
+        self.inputdata = inputdata
 
         # Current working data. Validators that operate on compound data types
         # (eg DictValidator, ListValidator) should use this to store
@@ -98,14 +98,18 @@ class Context(object):
         # Current fieldname being processed
         self.currentfield = None
 
-    def push(self, validator, rawdata=None, currentfield=None):
+    def push(self, validator, inputdata=None, currentfield=None, workingdata=None):
         """
         Create and return a new validation context with ``self`` as parent
         """
-        rawdata = self.rawdata if rawdata is None else rawdata
+        if not validator.creates_context:
+            return self
+
+        inputdata = self.inputdata if inputdata is None else inputdata
         currentfield = self.currentfield if currentfield is None else currentfield
-        context = Context(rawdata, validator, self)
+        context = Context(validator, inputdata, self)
         context.currentfield = currentfield
+        context.workingdata = workingdata
         return context
 
     def closest(self, type_):
@@ -113,8 +117,6 @@ class Context(object):
         Return the closest validation context associated with the given
         validator type.
         """
-        assert type_.creates_context, \
-                "Finding a context for %r makes no sense" % (type_,)
         context = self
         while context.parent:
             if isinstance(context.validator, type_) and context.validator.creates_context:
@@ -307,7 +309,8 @@ class BaseValidator(object):
         if value is Undefined:
             return Undefined
         for validator in self._subvalidators:
-            value = validator.process(value, context.push(self, value))
+            subcontext = context.push(self, value)
+            value = validator.process(value, subcontext)
         return value
 
     def __call__(self, *args, **kwargs):
@@ -444,7 +447,10 @@ class MappingValidator(BaseValidator):
         Add the given keyword arguments as field validators
         """
         for validator in subvalidators:
-            self.__setitem__(validator.source, make_validator(validator))
+            if validator.source is None:
+                self._subvalidators += [(None, validator)]
+            else:
+                self.__setitem__(validator.source, make_validator(validator))
 
         for name, validator in kwargs.items():
             self.__setitem__(name, make_validator(validator))
@@ -485,8 +491,7 @@ class MappingValidator(BaseValidator):
             value = {}
 
         errors = []
-        subcontext = context.push(self, value)
-        subcontext.workingdata = {}
+        subcontext = context.push(self, value, workingdata={})
 
         # tuples are hashable and so can be compared for loop detection
         to_process = tuple(self._subvalidators[:])
@@ -649,12 +654,16 @@ class ListValidator(BaseValidator):
     def process(self, value, context=None):
         if value is Undefined:
             value = []
+        if context is None:
+            context = Context(self, value)
         value = super(ListValidator, self).process(value, context)
         result = []
         errors = []
+        subcontext = context.push(self)
         for ix, item in enumerate(value):
             try:
-                value = self.itemtype.process(item)
+                subcontext.currentfield = ix
+                value = self.itemtype.process(item, subcontext)
                 if value is Undefined:
                     value = None
                 result.append(value)
@@ -734,7 +743,7 @@ class Using(BaseValidator):
 
     Example::
 
-        >>> from formalize import Using
+        >>> from formalize import FormValidator, Unicode, Using
         >>> from formalize.checks import assert_true
         >>> def check_password(p1, p2, message=u"Passwords do not match"):
         ...     assert_true(p1 == p2, message)
@@ -751,6 +760,8 @@ class Using(BaseValidator):
 
     """
 
+    creates_context = True
+
     def __init__(self, *source_fields):
         self.source_fields = source_fields
         super(Using, self).__init__()
@@ -758,8 +769,8 @@ class Using(BaseValidator):
     def process(self, value, context=None):
         if context is None:
             context = Context(self, value)
-        c = context.closest(DictValidator)
-        assert c is not None, "Using requires a DictValidator parent"
+        c = context.closest(MappingValidator)
+        assert c is not None, "Using requires a MappingValidator parent"
 
         try:
             value = tuple(
@@ -798,6 +809,8 @@ class Group(MappingValidator):
     type, specify these on the source fields as desired.
     """
 
+    creates_context = True
+
     def __init__(
         self,
         source_fields,
@@ -828,8 +841,8 @@ class Group(MappingValidator):
         Return processed input data from ``value`` or raise a
         ``ValidationError`` """
 
-        if not isinstance(self.parent, DictValidator):
-            raise AssertionError("Group object must be a child of a DictValidator")
+        if not isinstance(self.parent, MappingValidator):
+            raise AssertionError("Group object must be a child of a MappingValidator")
 
         try:
             dict_result = super(Group, self).process_subvalidators(value, context)
@@ -884,7 +897,7 @@ class Group(MappingValidator):
 
 class Case(MappingValidator):
     """
-    When objects work in the context of ``DictValidator`` objects,
+    When objects work in the context of ``MappingValidator`` objects,
     conditionally applying validation rules.
     """
 
@@ -923,13 +936,13 @@ class Case(MappingValidator):
                     return False
             return True
 
-        context = context.closest(DictValidator)
+        context = context.closest(MappingValidator)
 
         for case in self.cases:
             if case.applies(context):
-                return case.process_subvalidators(context.rawdata, context)
+                return case.process_subvalidators(context.workingdata, context)
 
-        return self.otherwise.process_subvalidators(context.rawdata, context)
+        return self.otherwise.process_subvalidators(context.workingdata, context)
 
 
         case_validator = DictValidator()
@@ -947,13 +960,12 @@ class WhenBase(MappingValidator):
         raise NotImplentedError
 
     def process_subvalidators(self, value, context):
-        context = context.closest(DictValidator)
+        context = context.closest(MappingValidator)
 
         if self.applies(context):
             context.workingdata.update(
-                super(WhenBase, self).process_subvalidators(context.rawdata, context)
+                super(WhenBase, self).process_subvalidators(context.inputdata, context)
             )
-            return Undefined
         return Undefined
 
 
